package org.mmga.hnyrespack.commands;

import com.mojang.brigadier.arguments.StringArgumentType;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.minecraft.network.packet.s2c.common.ResourcePackSendS2CPacket;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import net.minecraft.util.Formatting;
import org.mmga.hnyrespack.entities.ResPack;
import org.mmga.hnyrespack.manager.PackManager;

import java.io.IOException;
import java.net.URISyntaxException;
import java.security.NoSuchAlgorithmException;
import java.util.Optional;
import java.util.UUID;

import static net.minecraft.server.command.CommandManager.*;
import static org.mmga.hnyrespack.utils.Tools.onThreadException;
import static org.mmga.hnyrespack.utils.Tools.sendMessage;

public class ResPackCommand {

    public static void register() {
        CommandRegistrationCallback.EVENT.register(((dispatcher, registryAccess, environment) -> dispatcher.register(literal("resPack")
                        .requires(ServerCommandSource::isExecutedByPlayer)
                        .then(literal("download")
                                .requires(serverCommandSource -> serverCommandSource.hasPermissionLevel(2))
                                .then(argument("fileName", StringArgumentType.string())
                                        .executes(context -> {
                                            final String fileName = StringArgumentType.getString(context, "fileName");
                                            ServerPlayerEntity player = context.getSource().getPlayer();
                                            try {
                                                PackManager.packDownloader(fileName);
                                            } catch (URISyntaxException | IOException | NoSuchAlgorithmException e) {
                                                onThreadException(e, player);
                                                return 1;
                                            }
                                            sendMessage(player, "资源包" + fileName + "已下载完成", Formatting.GREEN);
                                            return 1;
                                        })))
                        .then(literal("remove")
                                .requires(serverCommandSource -> serverCommandSource.hasPermissionLevel(2))
                                .executes(context -> {
                                    ServerPlayerEntity player = context.getSource().getPlayer();
                                    PackManager.deleteAllPacks(player);
                                    sendMessage(player, "已删除全部资源包文件", Formatting.GREEN);
                                    return 1;
                                })
                                .then(argument("packName", StringArgumentType.string())
                                        .executes(context -> {
                                            ServerPlayerEntity player = context.getSource().getPlayer();
                                            final String packName = StringArgumentType.getString(context, "packName");
                                            PackManager.packDeleter(player, packName);
                                            sendMessage(player, "已删除资源包文件" + packName, Formatting.GREEN);
                                            return 1;
                                        })))
                        .then(literal("setPack")
                                .then(argument("packName", StringArgumentType.string())
                                        .executes(context -> {
                                            final String packName = StringArgumentType.getString(context, "packName");
                                            ServerPlayerEntity player = context.getSource().getPlayer();
                                            ResPack pack = PackManager.PACKS_HASH_MAP.get(packName);
                                            player.networkHandler.sendPacket(
                                                    new ResourcePackSendS2CPacket(
                                                            UUID.randomUUID(), pack.url(), pack.hash(), false, Optional.of(Text.of("建议安装此资源包以获得更好的游戏体验"))
                                                    )
                                            );
                                            return 1;
                                        }))
                        )
                )
                )
        )
        ;
    }
}
