package org.mmga.hnyrespack.manager;

import net.minecraft.server.network.ServerPlayerEntity;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.mmga.hnyrespack.entities.ResPack;

import java.io.*;
import java.net.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.stream.Stream;

import static org.mmga.hnyrespack.HnyResPack.packs;
import static org.mmga.hnyrespack.utils.Tools.onThreadException;

public class PackManager {

    private static final String BASE_URL = "https://wzpmc.cn:83/";
    private static final Path path = packs.toPath();

    public static final HashMap<String, ResPack> PACKS_HASH_MAP = new HashMap<>();
    private static final Logger log = LogManager.getLogger(PackManager.class);

    /**
     * 下载资源包
     * @param name 资源包文件名。注意：需包括后缀名
     */
    public static void packDownloader(String name) throws URISyntaxException, IOException, NoSuchAlgorithmException {
        packsDir();
        String s = BASE_URL + name;
        URI uri = new URI(s);
        URL url = uri.toURL();
        URLConnection urlConnection = url.openConnection();
        InputStream inputStream = urlConnection.getInputStream();
        try (FileOutputStream fileOutputStream = new FileOutputStream(path.resolve(name).toFile())) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = inputStream.read(buffer)) != -1) {
                fileOutputStream.write(buffer, 0, read);
            }
        }
        File downloadedFile = path.resolve(name).toFile();
        String sha1 = getFileSHA1(downloadedFile);
        PACKS_HASH_MAP.put(name, new ResPack(s, sha1));
    }

    public static void deleteAllPacks(ServerPlayerEntity player){
        Path path = packs.toPath();
        try {
            try(Stream<Path> walk = Files.walk(path)){
                walk.filter(Files::isRegularFile)
                        .forEach(filePath -> {
                            try {
                                Files.delete(filePath);
                                PACKS_HASH_MAP.clear();
                            } catch (IOException e) {
                                onThreadException(e, player);
                            }
                        });
            }
        } catch (IOException e) {
            onThreadException(e, player);
        }
    }

    public static void deleteAllPacks(){
        Path path = packs.toPath();
        try {
            try(Stream<Path> walk = Files.walk(path)){
                walk.filter(Files::isRegularFile)
                        .forEach(filePath -> {
                            try {
                                Files.delete(filePath);
                                PACKS_HASH_MAP.clear();
                            } catch (IOException e) {
                                log.catching(e);
                            }
                        });
            }
        } catch (IOException e) {
            log.catching(e);
        }
    }

    public static void packDeleter(ServerPlayerEntity player, String name){
        Path path = packs.toPath();
        try {
            try(Stream<Path> walk = Files.walk(path)){
                walk.filter(Files::isRegularFile)
                        .forEach(filePath -> {
                            if (filePath.getFileName().toString().equals(name)) {
                                try {
                                    Files.delete(filePath);
                                    PACKS_HASH_MAP.remove(name);
                                } catch (IOException e) {
                                    onThreadException(e, player);
                                }
                            }
                        });
            }
        } catch (IOException e) {
            onThreadException(e, player);
        }
    }

    private static void packsDir() {
        if (!packs.exists()) {
            //noinspection ResultOfMethodCallIgnored
            packs.mkdirs();
        }
    }

    private static String getFileSHA1(File file) throws IOException, NoSuchAlgorithmException {
        // 使用try-with-resources自动关闭资源
        try (FileInputStream fis = new FileInputStream(file)) {
            // 创建MessageDigest实例，指定SHA-1算法
            MessageDigest digest = MessageDigest.getInstance("SHA-1");

            // 创建字节缓冲区
            byte[] byteArray = new byte[1024];
            int bytesCount;

            // 读取文件并更新哈希计算
            while ((bytesCount = fis.read(byteArray)) != -1) {
                digest.update(byteArray, 0, bytesCount);
            }

            // 完成哈希计算并获取字节数组结果
            byte[] bytes = digest.digest();

            // 将字节数组转换为十六进制字符串
            StringBuilder sb = new StringBuilder();
            for (byte b : bytes) {
                sb.append(String.format("%02x", b));
            }

            // 返回SHA-1哈希字符串
            return sb.toString();
        }
    }
}
